import numpy as np
import matplotlib.pyplot as plt

# hustota site
N=40
xmax=20.0
dx=xmax/N

# geometrie kondenzatoru
L=5.0
D=3.0

# indexy koncu desek
i1=int((xmax/2-0.5*L)/dx)
i2=int((xmax/2+0.5*L)/dx)
j1=int((xmax/2-0.5*D)/dx)
j2=int((xmax/2+0.5*D)/dx)

u=np.zeros((N+1,N+1))
u[i1:i2+1,j1]=-1 # zaporna deska
u[i1:i2+1,j2]=+1 # kladna deska

# relaxacni faktor
w=2/(1+np.pi/N)

# Gaussova-Seidelova iteracni metoda
for k in range(2000):
    maxdev=0
    for i in range(1,N):
        for j in range(1,N):
            if (i1<=i and i<=i2) and (j==j1 or j==j2) : continue
            unew=(u[i-1,j]+u[i+1,j]+u[i,j-1]+u[i,j+1])/4
            maxdev=np.fmax(maxdev,np.abs(unew-u[i,j]))
            u[i,j]=w*(unew-u[i,j])+u[i,j]

    # okrajove podminky 1 - nulovy potencial
    u[0,:]=0.0
    u[N,:]=0.0
    u[:,0]=0.0
    u[:,N]=0.0
    # okrajove podminky 2 - pole rovnobezne s okrajem
    #u[0,:]=u[1,:]
    #u[N,:]=u[N-1,:]
    #u[:,0]=u[:,1]
    #u[:,N]=u[:,N-1]

    if maxdev<1e-5 : break
    if k%10==0 : print(k,maxdev)

# graf reseni - ekvipotencialy a desky
x=np.linspace(0,xmax,N+1)
lvl=np.linspace(-1,+1,20)
plt.contourf(x,x,np.transpose(u),levels=lvl)
plt.contour(x,x,np.transpose(u),levels=lvl,colors='k',linestyles='solid')
plt.hlines(y=[xmax/2-D/2,xmax/2+D/2],xmin=xmax/2-L/2,xmax=xmax/2+L/2,linewidth=5)     
plt.show()
