import numpy as np
import matplotlib.pyplot as plt
import scipy.sparse
import scipy.sparse.linalg

# hustota site
N=100
xmax=20.0
dx=xmax/N

# geometrie kondenzatoru
L=5.0
D=3.0

# indexy koncu desek
i1=int((xmax/2-0.5*L)/dx)
i2=int((xmax/2+0.5*L)/dx)
j1=int((xmax/2-0.5*D)/dx)
j2=int((xmax/2+0.5*D)/dx)

# sestaveni matice systemu a vektoru pravych stran

Neq=(N+1)**2

row=[]
col=[]
val=[]
rhs=np.zeros(Neq)

def indx(i,j):
    return (N+1)*i+j

def add(r,c,v):
    row.append(r)
    col.append(c)
    val.append(v)

for i in range(0,N+1):
    for j in range(0,N+1):
        r=indx(i,j)
        if i==0 or i==N or j==0 or j==N :
            # okraj oblasti
            add(r,r,1.0)
            rhs[r]=0.0
        elif (i1<=i and i<=i2) and (j==j1 or j==j2) :
            # deska
            add(r,r,1.0)
            if j==j1: rhs[r]=-1.0
            if j==j2: rhs[r]=+1.0
        else:
            # vnitrni bod
            add(r,r          ,-4.0)
            add(r,indx(i-1,j), 1.0)
            add(r,indx(i+1,j), 1.0)
            add(r,indx(i,j-1), 1.0)
            add(r,indx(i,j+1), 1.0)
            rhs[r]=0.0

# reseni ridkeho systemu linearnich rovnic
mtx=scipy.sparse.csc_matrix( (val, (row, col)), shape=(Neq,Neq) )
sol=scipy.sparse.linalg.spsolve(mtx,rhs)

# extrakce reseni do 2D pole
u=np.zeros((N+1,N+1))
for i in range(0,N+1):
    for j in range(0,N+1):
        u[i,j]=sol[indx(i,j)]

u=np.reshape(sol,(N+1,N+1))

# graf reseni - ekvipotencialy a desky
x=np.linspace(0,xmax,N+1)
lvl=np.linspace(-1,+1,20)
plt.contourf(x,x,np.transpose(u),levels=lvl)
plt.contour(x,x,np.transpose(u),levels=lvl,colors='k',linestyles='solid')
plt.hlines(y=[xmax/2-D/2,xmax/2+D/2],xmin=xmax/2-L/2,xmax=xmax/2+L/2,linewidth=5)     
plt.show()
